<?php
// $Id: flashclient.config.inc,v 1.1.2.1 2008/08/12 07:26:18 openwereld Exp $

class FlamingoConfig {
  var $dom;
  var $root;
  var $map_id;
  
  function __construct() {
    $this->dom = new DOMDocument('1.0', 'UTF-8');
    $this->root = $this->dom->createElement('FLAMINGO');
    $this->dom->appendChild($this->root);
    $this->setAttributes($this->root, array('xmlns:fmc' => 'fmc'
      , 'lang' => 'en'
      , 'tooltipdelay' => '500'
      , 'maxwidth' => '900'
      , 'maxheight' => '700'
      , 'borderwidth' => '0'
      , 'borderalpha' => '0'
      ));
    
    // add logo
    $element = $this->root->appendChild($this->dom->createElement('fmc:Logo'));
    $element->setAttribute('id', 'logo');
    
    // default map id
    $this->map_id = 'map1';
    
    $this->addButtons();
  }
  
  /**
   *
   */
  function setAttributes($element, $attributes) {
    foreach ($attributes as $key => $value) {
      $element->setAttribute($key, $value);
    }
    return $element;
  }
  
  /**
   *
   */
  function addButtons() {
    $group = $this->dom->createElement('fmc:ToolGroup');
    $this->setAttributes($group, array('listento' => $this->map_id
      , 'left' => "0"
      , 'top' => "0"
      , 'tool' => "pan"
      ));
    $this->root->appendChild($group);
    
    // zoom/pan in group (select 1 one group; pan is default)
    $element = $group->appendChild($this->dom->createElement('fmc:ToolZoomin'));
    $this->setAttributes($element, array('left' => "0", 'zoomdelay' => "0", 'clickdelay' => "0"));
    $element = $group->appendChild($this->dom->createElement('fmc:ToolZoomout'));
    $this->setAttributes($element, array('left' => "30", 'zoomdelay' => "0", 'clickdelay' => "0"));
    $element = $group->appendChild($this->dom->createElement('fmc:ToolPan'));
    $this->setAttributes($element, array('id' => "pan", 'left' => "60", 'clickdelay' => "0", 'pandelay' => "0"));

    // additional buttons for zoom history and full extent
    $element = $this->root->appendChild($this->dom->createElement('fmc:ButtonPrev'));
    $this->setAttributes($element, array('left' => 90, 'top' => 0, 'listento' => $this->map_id));
    $element = $this->root->appendChild($this->dom->createElement('fmc:ButtonNext'));
    $this->setAttributes($element, array('left' => 120, 'top' => 0, 'listento' => $this->map_id));
    $element = $this->root->appendChild($this->dom->createElement('fmc:ButtonFull'));
    $this->setAttributes($element, array('left' => 150, 'top' => 0, 'listento' => $this->map_id));
    
    // add coordinates
    $coordinates = $this->root->appendChild($this->dom->createElement('fmc:Coordinates'));
    $this->setAttributes($coordinates, array('left' => "right -180"
      , 'top' => "bottom -14"
      , 'decimals' => "2"
      , 'listento' => $this->map_id
      ));
    $element = $coordinates->appendChild($this->dom->createElement('style'));
    $this->setAttributes($element, array('id' => ".xy"
      , 'font-family' => "verdana"
      , 'font-size' => "10px"
      , 'color' => "#555555"
      , 'display' => "block"
      , 'font-weight' => "normal"
      ));
    $element = $coordinates->appendChild($this->dom->createElement('string'));
    $this->setAttributes($element, array('id' => 'xy'));
    $element->appendChild($this->dom->createElement('en', 'x:[x]  y:[y]'));
  }
  
  /**
   * @param $node
   *   Drupal node (wms or map context).
   */
  function addMap($node) {
    $map = $this->root->appendChild($this->dom->createElement('fmc:Map'));
    $this->setAttributes($map, array('id' => $this->map_id
      , 'extenthistory' => "10"
      , 'left' => "0", 'top' => "0"
      , 'bottom' => "bottom"
      , 'right' => "right"
      , 'visible' => "true"
      ));
    
    $element = $map->appendChild($this->dom->createElement('fmc:LayerIdentifyIcon'));
    $this->setAttributes($element, array('id' => 'identifyicon'));
    
    $element = $map->appendChild($this->dom->createElement('fmc:LayerGrid'));
    $this->setAttributes($element, array('id' => 'grid', 'gridwidth' => 10, 'gridheight' => 10));
    
    $count = 0;
    if ($node->type == 'mapcontext') {
      $fullextent = $node->minx .','. $node->miny .','. $node->maxx .','. $node->maxy;
      if (isset($node->dminx)) {
        $extent = $node->dminx .','. $node->dminy .','. $node->dmaxx .','. $node->dmaxy;
      }
      $srs = 'EPSG:'. $node->srid;
      
      foreach ($node->layers as $layer) {
        $element = $map->appendChild($this->dom->createElement('fmc:LayerOGWMS'));
        $this->setAttributes($element, array('id' => 'OG'.$count++
          , 'url' => mappingkit_get_proxy_url($layer['href'] .'&SERVICE=WMS')
          , 'srs' => $srs
          , 'layers' => $layer['names']
          , 'exceptions' => 'inimage'
          , 'version' => $layer['version']
          ));
      }
    }
    else if ($node->type == 'wms') {
      //$map->appendChild($this->dom->createComment(var_export($node, true)));
      $boundingbox = $node->layers[0]->boundingbox[$node->wms_srs];
      $fullextent = $boundingbox['minx'] .','. $boundingbox['miny'] .','. $boundingbox['maxx'] .','. $boundingbox['maxy'];
      
      $element = $map->appendChild($this->dom->createElement('fmc:LayerOGWMS'));
      $this->setAttributes($element, array('id' => 'OG'.$count++
        , 'url' => mappingkit_get_proxy_url($node->mapurl)
        , 'getcapabilitiesurl' => mappingkit_get_proxy_url($node->serviceurl)
        , 'srs' => $node->wms_srs
        , 'layers' => '#ALL#'
        ));
    }
    else {
      $element = $map->appendChild($this->dom->createElement('fmc:LayerOGWMS'));
      $this->setAttributes($element, array('id' => "OG1"
        , 'format' => "GIF"
        , 'url' => mappingkit_get_proxy_url("http://www2.demis.nl/wms/wms.asp?WMS=WorldMap&amp;VERSION=1.1.1&amp;SRS=EPSG:4326&amp;EXCEPTIONS=INIMAGE&amp;BGCOLOR=0xF0F0F0")
        , 'QUERY_LAYERS' => "Cities,Settlements,Airports"
        , 'LAYERS' => "Builtup areas,Coastlines,Waterbodies,Inundated,Rivers,Streams,Railroads,Highways,Roads,Trails,Borders,Cities,Settlements,Spot elevations,Airports,Ocean features"
        ));
      $element = $map->appendChild($this->dom->createElement('fmc:LayerOGWMS'));
      $this->setAttributes($element, array('id' => "OG2"
        , 'format' => "PNG"
        , 'url' => mappingkit_get_proxy_url("http://www2.demis.nl/wms/wms.asp?WMS=WorldMap&amp;VERSION=1.1.1&amp;SRS=EPSG:4326&amp;EXCEPTIONS=INIMAGE&amp;BGCOLOR=0xF0F0F0")
        , 'LAYERS' => "Bathymetry,Countries,Topography,Hillshading"
        ));
    }
    
    if (!$extent) {
      $extent = $fullextent;
    }
    $this->setAttributes($map, array('extent' => $extent
      , 'fullextent' => $fullextent
      ));
  }
  
  /**
   *
   */
  function saveXML() {
    $this->dom->formatOutput = TRUE;
    return $this->dom->saveXML();
  }
}

/**
 * @param $node
 *   Drupal node for which Flamingo config document must be generated.
 */
function flashclient_context_config($node) {
  $config = new FlamingoConfig();
  
  $config->addMap($node);
  
  drupal_set_header('Content-Type: text/xml; charset=utf-8');
  echo $config->saveXML();
  return;
}

