<?php
// $Id: xml.parser.inc,v 1.1.2.2 2008/08/16 10:48:07 openwereld Exp $

class XML_Parser {
  public $namespaces;
  public $doc;
  public $root;
  public $content;
  public $defaultNamespacePrefix;
  
  function __construct() {
    $this->namespaces = array(
      'xlink'=> "http://www.w3.org/1999/xlink",
      'xsi'=> "http://www.w3.org/2001/XMLSchema-instance"
      );
    
    $this->defaultNamespacePrefix = NULL;
  }
  
  function getNamespacePrefix($uri = NULL) {
    if (!$uri) {
      return $this->defaultNamespacePrefix; // default namespace prefix
    }
    else {
      foreach ($this->namespaces as $prefix => $namespace) {
        if ($namespace == $uri) {
          return $prefix;
        }
      }
    }
    return NULL;
  }
  
  function get_child_value($node, $default = '') {
    $value = $node->firstChild->nodeValue;
    return $value ? $value : $default;
  }
  
  function read_property($name, &$context, $node) {
    $value = $this->get_child_value($node);
  //  drupal_set_message(var_export(array('tag'=>$node->tagName, 'nodeValue' => $node->nodeValue, 'value' => $value),true));
    if ($value) {
      $context[$name] = $value;
    }
  }
  
  function run_child_nodes(&$obj, $node) {
    foreach ($node->childNodes as $childNode) {
      if ($childNode->nodeType == 1) {
        $prefix = $this->getNamespacePrefix($childNode->namespaceURI);
        $local = array_pop(explode(':', $childNode->nodeName));
        $function = 'read_'. $prefix .'_'. $local;
        if (method_exists($this, $function)) {
          $this->$function($obj, $childNode);
        }
        else if (function_exists($function)) {
          $function($obj, $childNode);
        }
      }
    }
  }
  
  function load($xml) {
    $this->doc = new DOMDocument('1.0', 'UTF-8');
    if (! $this->doc->loadXML($xml)) {
      drupal_set_message(t('Failed to load xml'));
      return NULL;
    }
    $this->doc->formatOutput = TRUE;
    $this->root = $this->doc->documentElement;
    $this->content = array('version' => $this->root->getAttribute('version'));
    $this->run_child_nodes($this->content, $this->root);
    return $this->content;
  }
}

