// $Id: GPX.js,v 1.1.2.4 2008/06/08 08:01:34 openwereld Exp $

/* Copyright (c) 2006-2007 MetaCarta, Inc., published under a modified BSD license.
 * See http://svn.openlayers.org/trunk/openlayers/repository-license.txt 
 * for the full text of the license. */

/**
 * @requires OpenLayers/Format/XML.js
 * @requires OpenLayers/Feature/Vector.js
 * @requires OpenLayers/Geometry/Point.js
 * @requires OpenLayers/Geometry/LineString.js
 * @requires OpenLayers/Geometry/Polygon.js
 *
 * Class: OpenLayers.Format.GPX
 * Read/write GPX parser. Create a new instance with the 
 *     <OpenLayers.Format.GPX> constructor.
 *
 * Inherits from:
 *  - <OpenLayers.Format.XML>
 */
OpenLayers.Format.GPX = OpenLayers.Class(OpenLayers.Format.XML, {
    
    /**
     * Constructor: OpenLayers.Format.GPX
     * Create a new parser for GPX.
     *
     * Parameters:
     * options - {Object} An optional object whose properties will be set on
     *     this instance.
     */
    initialize: function(options) {
        OpenLayers.Format.XML.prototype.initialize.apply(this, [options]);
    },
    
    /**
     * APIMethod: read
     * Return a list of features from a GPX doc
     
     * Parameters:
     * data - {Element} 
     *
     * Returns:
     * An Array of <OpenLayers.Feature.Vector>s
     */
    read: function(doc) {
        if (typeof doc == "string") { 
            doc = OpenLayers.Format.XML.prototype.read.apply(this, [doc]);
        }
        var features = [];
        
        var tracks = doc.getElementsByTagName("trk");
        for (var i = 0; i < tracks.length; i++) {
            var segs = this.getElementsByTagNameNS(tracks[i], tracks[i].namespaceURI, "trkseg");
            for (var j = 0; j < segs.length; j++) {
                var track = this.extractSegment(segs[j], "trkpt");
                features.push(track);
            }
        }
        
        var routes = doc.getElementsByTagName("rte");
        for (var k = 0; k < routes.length; k++) {
            var route = this.extractSegment(routes[k], "rtept");
            features.push(route);
        }
        
        var waypoints = doc.getElementsByTagName("wpt");
        for (var i = 0; i < waypoints.length; i++) {
            // note: this adds a point geometry, not a feature with a geometry
            features.push(new OpenLayers.Geometry.Point(waypoints[i].getAttribute("lon"), waypoints[i].getAttribute("lat")));
        }
        
        if (this.internalProjection && this.externalProjection) {
            for (var g = 0; g < features.length; g++) {
                if (features[g].geometry) {
                      features[g].geometry.transform(this.externalProjection,
                          this.internalProjection);
                }
            }
        }
        
        return features;
    },
    
    extractSegment: function(segment, segmentType) {
        var points = this.getElementsByTagNameNS(segment, segment.namespaceURI, segmentType);
        var point_features = [];
        for (var i = 0; i < points.length; i++) {
            point_features.push(new OpenLayers.Geometry.Point(points[i].getAttribute("lon"), points[i].getAttribute("lat")));
        }
        return new OpenLayers.Feature.Vector(new OpenLayers.Geometry.LineString(point_features));
    },
    
    CLASS_NAME: "OpenLayers.Format.GPX" 
});

