// $Id: Legend.js,v 1.1.2.1 2008/06/06 08:02:09 openwereld Exp $

/**
 * @requires OpenLayers/Util.js
 * @requires OpenLayers.Events.js
 */

/**
 * Class: MappingKit.Legend
 */
MappingKit.Legend = OpenLayers.Class({
    
    /**
     * APIProperty: div
     * {DOMElement} The element that contains the legend
     */
    div: null,
    
    /**
     * APIProperty: map
     * {DOMElement} The map object
     */
    map: null,
    
    /**
     * Constructor: MappingKit.Legend
     * 
     * Parameters:
     * map - {Object}
     * div - {string}
     * options - {Object}
     */
    initialize: function(map, div, options) {
        // set defaults
        
        // now override default options
        OpenLayers.Util.extend(this, options);
        
        // set div
        this.div = OpenLayers.Util.getElement(div);
        
        // link to map
        if(!(map instanceof OpenLayers.Map)) {
            map = MappingKit.getMap(map);
        }
        this.setMap(map);
    },
    
    /** 
     * Method: setMap
     *
     * Properties:
     * map - {<OpenLayers.Map>} 
     */
    setMap: function(map) {
        this.map = map;
        
        this.map.events.on({
            'addlayer': this.redraw,
            'changelayer': this.redraw,
            'removelayer': this.redraw,
            'changebaselayer': this.redraw,
            scope: this
        });
    },
    
    redraw: function() {
        if (!this.div) {
            return;
        }
      
        var layers = this.map.layers;
        // clear legend
        for (var i = this.div.childNodes.length - 1; i >= 0; i--) {
            this.div.removeChild(this.div.childNodes[i]);
        }
        for (var i = 0; i < layers.length; i++) {
            var layer = layers[i];
            if (layer.displayInLegend && layer.visibility && layer.legend) {
                
                var layerElem = document.createElement('div');
                
                var titleElem = document.createElement('p');
                titleElem.innerHTML = layer.name;
                titleElem.className = 'legend-title';
                layerElem.appendChild(titleElem);
                
                var imgElem = document.createElement('img');
                imgElem.src = layer.legend.href;
                imgElem.title = layer.name;
                layerElem.appendChild(imgElem);
                
                this.div.appendChild(layerElem);
            }
        }
        
        return this.div;
    },
    
    CLASS_NAME: "MappingKit.Legend"
});

