// $Id: openlayers.js,v 1.1.2.21 2008/10/31 14:25:52 openwereld Exp $

MappingKit.OpenLayers = {};
MappingKit.selectControls = [];

if (!$defined(MappingKit.Control)) {
  MappingKit.Control = {};
}
if (!$defined(MappingKit.Format)) {
  MappingKit.Format = {};
}

if (!$defined(MappingKit.Layer)) {
  MappingKit.Layer = {};
}

MappingKit.calculateInRange = function() {
  var inRange = false;
  if (this.map) {
    var resolution = this.map.getResolution();
    inRange = ( (resolution >= this.minResolution) &&
                (resolution <= this.maxResolution) );
    var extent = this.map.getExtent();
    if (extent) {
      // check that maxExtent of layer is within the current extent
      inRange = inRange && extent.intersectsBounds(this.maxExtent);
    }
  }
  return inRange;
}

/**
 * custom translate function for OpenLayers to use Drupal language texts
 */
MappingKit.translate = function(key, context) {
  var en_dictionary = OpenLayers.Lang['en'];
  var message = en_dictionary[key];
  if (OpenLayers.Lang.getCode() != 'en') {
    // check Drupal's translation
    var drupal_message = Drupal.t(key);
    if (drupal_message != key) {
      // Drupal should return something different from the key
      message = drupal_message;
    }
  }
  if (!message) {
    message = key;
  }
  if (context) {
    message = OpenLayers.String.format(message, context);
  }
  return message;
}

// redirect OpenLayers language support to Drupal
OpenLayers.Lang.translate = MappingKit.translate;
OpenLayers.i18n = MappingKit.translate;

MappingKit.OpenLayers.onFeatureModification = function(feature) {
  var el = document.getElementById('edit-geometry');
  if (el) {
    el.value = feature.geometry;
  }
}

// baseLayer
// overlays
// permalink
// scale

MappingKit.movestart = function(evt) {
  if (!evt.dragging) {
    // reset array
    MappingKit.loadErrors = [];
    
    var el = document.getElementById('mapclient-messages');
    if (el) {
      // remove list item
      $(el).empty();
      
      // remove error class from parent
      $(el).parent().removeClass('error');
    }
  }
}

MappingKit.removelayer = function(evt) {
  if (!MappingKit.loadErrors) {
    MappingKit.loadErrors = [];
  }
  if (evt.layer instanceof OpenLayers.Layer.WMS) {
    for (var i = 0; i < MappingKit.loadErrors.length; i++) {
      if (evt.layer.params && MappingKit.loadErrors[i] == evt.layer.params.LAYERS) {
        //alert('remove ' + evt.layer.name);
        MappingKit.loadErrors.splice(i, 1);
        var item = document.getElementById('message-' + evt.layer.params.LAYERS);
        if (item) {
          item.parentNode.removeChild(item);
        }
        return;
      }
    }
  }
}

MappingKit.onImageLoadError = function() {
  if (!MappingKit.loadErrors) {
    MappingKit.loadErrors = [];
  }
  var el = document.getElementById('mapclient-messages');
  if (el) {
    var parts = this.src.split('?');
    var request = {service: parts[0]};
    if (parts.length > 1) {
      var params = parts[1].split('&');
      for (var i = 0; i < params.length; i++) {
        var tmp = params[i].split('=');
        var key = tmp[0].toLowerCase();
        var value = unescape(tmp[1]);
        request[key] = value;
        if (key == 'layers') {
          var layers = unescape(tmp[1]);
          
          // check if layer error is already shown
          for (var j = 0; j < MappingKit.loadErrors.length; j++) {
            if (MappingKit.loadErrors[j] == layers) {
              return;
            }
          }
          if (j == 0) {
            $(el).parent().addClass('error');
          }
          MappingKit.loadErrors[j] = layers;
        }
      }
    }
    else if (request.service.indexOf('/404.png') >= 0) {
      // ignore this one
      return;
    }
    else {
      // not a wms layer request; only show host name part of service
      var matches = request.service.match(/http\:\/\/([^\/]*)\//);
      var layers = matches[0];
      // check if layer error is already shown
      for (var j = 0; j < MappingKit.loadErrors.length; j++) {
        if (MappingKit.loadErrors[j] == layers) {
          return;
        }
      }
      if (j == 0) {
        $(el).parent().addClass('error');
      }
      MappingKit.loadErrors[j] = layers;
    }
    var listItem = document.createElement('li');
    listItem.id = 'message-' + layers;
    listItem.innerHTML = Drupal.t('Error loading images for <em>@layers</em>.', {'@layers': layers});
    el.appendChild(listItem);
  }
}

MappingKit.onPopupClose = function(evt) {
  for (var i = 0; i < MappingKit.selectControls.length; i++) {
    if (MappingKit.selectControls[i].layer == MappingKit.selectedFeature.layer) {
      MappingKit.selectControls[i].unselect(MappingKit.selectedFeature);
      break;
    }
  }
}

MappingKit.wfsFeaturePopup = function(feature) {
  MappingKit.selectedFeature = feature;
  
  var content = '<div style="font-size:.75em;">';
  for (var item in feature.data) {
    content += item + '=' + feature.data[item] + '<br/>';
  }
  content += '</div>';
  //var content = "<div style='font-size:.8em'>Feature: " + feature.id +"<br />Area: " + feature.geometry.getArea()+"</div>";
  popup = new OpenLayers.Popup.FramedCloud("chicken", // id
    feature.geometry.getBounds().getCenterLonLat(), // lonlat
    null, // size
    content,
    null, // anchor
    true, // closeBox
    MappingKit.onPopupClose // closeBoxCallback
    );
  feature.popup = popup;
  this.map.addPopup(popup);
}

MappingKit.wfsFeatureUnPop = function(feature) {
  this.map.removePopup(feature.popup);
  feature.popup.destroy();
  feature.popup = null;
}

function osm_getTileURL(bounds) {
  var res = this.map.getResolution();
  
  var x = Math.round((bounds.left - this.maxExtent.left) / (res * this.tileSize.w));
  var y = Math.round((this.maxExtent.top - bounds.top) / (res * this.tileSize.h));
  var z = this.map.getZoom();
  var limit = Math.pow(2, z);

  if (y < 0 || y >= limit) {
    return OpenLayers.Util.getImagesLocation() + "404.png";
  } else {
    x = ((x % limit) + limit) % limit;
    return this.url + z + "/" + x + "/" + y + "." + this.type;
  }
}

MappingKit.onFeatureSelect = function(feature) {
  MappingKit.selectedFeature = feature;
  
  var content = '<div style="font-size:.8em;">';
  if (feature.attributes['name']) {
    content += '<h2>' + feature.attributes['name'] + '</h2>' + '<br/>';
  }
  if (feature.attributes['description']) {
    content += feature.attributes.description;
  }
  else {
    for (var item in feature.attributes) {
      if (item != 'name' && feature.attributes[item] != '') {
        content += item + '=' + feature.attributes[item] + '<br/>';
      }
    }
  }
  content += '</div>';
  
  popup = new OpenLayers.Popup.FramedCloud("chicken"
    , feature.geometry.getBounds().getCenterLonLat()
    , null // new OpenLayers.Size(100, 100)
    , content //'<h2>' + feature.attributes.name + '</h2>' + feature.attributes.desc
    , null
    , true // closebox
    , MappingKit.onPopupClose
    );
  feature.popup = popup;
  this.map.addPopup(popup);
}

MappingKit.onFeatureUnselect = function(feature) {
  this.map.removePopup(feature.popup);
  feature.popup.destroy();
  feature.popup = null;
}

